/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ExampleClient
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */


#pragma once


// ArmarX
#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/interface/observers/ObserverInterface.h>
#include <ArmarXCore/util/tasks.h>
#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>

// RobotAPI
#include <RobotAPI/interface/armem/server/MemoryInterface.h>
#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>
#include <RobotAPI/libraries/armem/client/ComponentPlugin.h>
#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>


namespace armarx
{
    /**
     * @class ExampleClientPropertyDefinitions
     * @brief Property definitions of `ExampleClient`.
     */
    class ExampleMemoryClientPropertyDefinitions :
        public armarx::ComponentPropertyDefinitions
    {
    public:
        ExampleMemoryClientPropertyDefinitions(std::string prefix);
    };


    /**
     * @defgroup Component-ExampleClient ExampleClient
     * @ingroup RobotAPI-Components
     * A description of the component ExampleClient.
     *
     * @class ExampleClient
     * @ingroup Component-ExampleClient
     * @brief Brief description of class ExampleClient.
     *
     * Detailed description of class ExampleClient.
     */
    class ExampleMemoryClient :
        virtual public armarx::Component,
        virtual public armarx::armem::client::ComponentPluginUser,
        virtual public LightweightRemoteGuiComponentPluginUser
    {
    public:

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;


        // LightweightRemoteGuiComponentPluginUser interface
    public:
        void createRemoteGuiTab();
        void RemoteGui_update() override;


    protected:

        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;
        void onExitComponent() override;

        void run();


    private:

        armem::MemoryID providerID;
        armem::MemoryID entityID;

        // Callback for updates on `example_entity`.
        void example_entityUpdated(const armem::MemoryID& id, const std::vector<armem::MemoryID>& snapshotIDs);

        // Examples
        void waitForMemory();
        armem::MemoryID addProviderSegment();

        armem::MemoryID commitSingleSnapshot(const armem::MemoryID& entityID);
        void commitMultipleSnapshots(const armem::MemoryID& entityID, int num = 3);

        void queryLatestSnapshot(const armem::MemoryID& entityID);
        void queryExactSnapshot(const armem::MemoryID& snapshotID);

        void commitExampleData();
        void queryExampleData();

    private:

        armarx::RunningTask<ExampleMemoryClient>::pointer_type task;

        armarx::DebugObserverInterfacePrx debugObserver;

        std::string memoryName = "Example";

        struct RemoteGuiTab : RemoteGui::Client::Tab
        {
            std::atomic_bool rebuild = false;

            std::optional<armem::wm::Memory> queryResult;
            RemoteGui::Client::GroupBox queryResultGroup;
        };
        RemoteGuiTab tab;
    };
}
