#pragma once

#include <string>
#include <vector>
#include <map>
#include <memory>

namespace armarx::armem::ltm
{

    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class MongoDBConnectionManager
    {
    public:
        struct MongoDBSettings
        {
            std::string host = "localhost";
            std::string user = "root";
            std::string password = "";

            bool isSet() const
            {
                // we always need a user and a host
                return !host.empty() and !user.empty();
            }

            std::string uniqueString() const
            {
                return host + "::" + user;
            }
        };

        static int EstablishConnection(const MongoDBSettings& settings)
        {
            const auto str_rep = settings.uniqueString();
            const auto& it = Connections.find(str_rep);
            if (it == Connections.end())
            {
                auto con = Connections.emplace(str_rep, 0);
                return con.first->second;
            }
            else
            {
                // A connection already exists. We do not need to open another one.
                return it->second;
            }
        }

    private:
        static std::map<std::string, int> Connections;
    };
}
