#pragma once

#include <RobotAPI/interface/armem/query.h>

#include "BaseQueryProcessorBase.h"
#include "EntityQueryProcessorBase.h"

#include <regex>

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/libraries/armem/core/error.h>


namespace armarx::armem::base::query_proc
{

    /**
     * @brief Handles memory queries.
     */
    template <class _ProviderSegmentT>
    class ProviderSegmentQueryProcessorBase :
        virtual public BaseQueryProcessorBase<_ProviderSegmentT, armem::query::data::ProviderSegmentQuery>
    {
        using Base = BaseQueryProcessorBase<_ProviderSegmentT, armem::query::data::ProviderSegmentQuery>;

    public:
        using ProviderSegmentT = _ProviderSegmentT;
        using EntityT = typename ProviderSegmentT::EntityT;
        using EntitySnapshotT = typename EntityT::EntitySnapshotT;

        using Base::process;
        void process(_ProviderSegmentT& result,
                     const armem::query::data::ProviderSegmentQuery& query,
                     const _ProviderSegmentT& providerSegment) const override
        {
            if (auto q = dynamic_cast<const armem::query::data::provider::All*>(&query))
            {
                process(result, *q, providerSegment);
            }
            else if (auto q = dynamic_cast<const armem::query::data::provider::Single*>(&query))
            {
                process(result, *q, providerSegment);
            }
            else if (auto q = dynamic_cast<const armem::query::data::provider::Regex*>(&query))
            {
                process(result, *q, providerSegment);
            }
            else
            {
                throw armem::error::UnknownQueryType("provider segment", query);
            }
        }

        void process(_ProviderSegmentT& result,
                     const armem::query::data::provider::All& query,
                     const _ProviderSegmentT& providerSegment) const
        {
            for (const auto& [name, entity] : providerSegment.entities())
            {
                result.addEntity(entityProcessorProcess(query.entityQueries, entity));
            }
        }

        void process(_ProviderSegmentT& result,
                     const armem::query::data::provider::Single& query,
                     const _ProviderSegmentT& providerSegment) const
        {
            try
            {
                const EntityT& entity = providerSegment.getEntity(query.entityName);
                result.addEntity(entityProcessorProcess(query.entityQueries, entity));
            }
            catch (const error::MissingEntry&)
            {
                // Leave empty.
            }
        }

        void process(_ProviderSegmentT& result,
                     const armem::query::data::provider::Regex& query,
                     const _ProviderSegmentT& providerSegment) const
        {
            std::regex regex(query.entityNameRegex);
            for (const auto& [name, entity] : providerSegment.entities())
            {
                if (std::regex_search(entity.name(), regex))
                {
                    result.addEntity(entityProcessorProcess(query.entityQueries, entity));
                }
            }
        }

    protected:
        virtual EntityT entityProcessorProcess(const armem::query::data::EntityQuerySeq& a, const EntityT& o) const = 0;
    };
}
