/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <mutex>
#include <optional>

#include "RobotAPI/libraries/armem/client.h"
#include "RobotAPI/libraries/armem/client/Reader.h"

#include "interfaces.h"

namespace armarx::armem::articulated_object
{
    class Reader:
        virtual public ReaderInterface
    // virtual public ::armarx::armem::MemoryConnector
    {
    public:
        Reader(armem::ClientReaderComponentPluginUser& component);
        virtual ~Reader() = default;

        void synchronize(ArticulatedObject& obj, const armem::Time& timestamp) override;

        std::optional<ArticulatedObject> get(const std::string& name, const armem::Time& timestamp) override;
        ArticulatedObject get(const ArticulatedObjectDescription& description, const armem::Time& timestamp) override;

        std::optional<robot::RobotState> queryState(const robot::RobotDescription& description, const armem::Time& timestamp);
        std::optional<robot::RobotDescription> queryDescription(const std::string& name, const armem::Time& timestamp);


        // TODO(fabian.reister): register property defs

    protected:
        std::optional<robot::RobotState> getRobotState(const armarx::armem::wm::Memory& memory) const;
        std::optional<robot::RobotDescription> getRobotDescription(const armarx::armem::wm::Memory& memory) const;

    private:

        struct Properties
        {
            std::string memoryName               = "Object";
            std::string coreInstanceSegmentName  = "ArticulatedObjectInstance";
            std::string coreClassSegmentName     = "ArticulatedObjectClass";
            std::string providerName;
        } properties;

        const std::string propertyPrefix = "mem.obj.articulated.";

        armem::client::Reader memoryReader;
        std::mutex memoryWriterMutex;

        armem::ClientReaderComponentPluginUser& component;
    };


}  // namespace armarx::armem::articulated_object