/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <mutex>

#include "RobotAPI/libraries/armem/client/Reader.h"
#include "RobotAPI/libraries/armem/client/Writer.h"
#include "RobotAPI/libraries/armem/client.h"

#include "interfaces.h"


namespace armarx::armem::articulated_object
{

    class Writer:
        virtual public WriterInterface //,
    // virtual public MemoryConnector
    {
    public:
        Writer(armem::ClientComponentPluginUser& component);
        virtual ~Writer() = default;


        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);
        void connect();


        bool store(const ArticulatedObject& obj) override;

        bool storeInstance(const ArticulatedObject& obj);
        std::optional<armem::MemoryID> storeClass(const ArticulatedObject& obj);

        // const std::string& getPropertyPrefix() const override;

    private:
        std::optional<armem::MemoryID> storeOrGetClass(const ArticulatedObject& obj);

        void updateKnownObjects(const armem::MemoryID& subscriptionID, const std::vector<armem::MemoryID>& snapshotIDs);
        void updateKnownObject(const armem::MemoryID& snapshotId);

        struct Properties
        {
            std::string memoryName              = "Object";
            std::string coreInstanceSegmentName = "ArticulatedObjectInstance";
            std::string coreClassSegmentName    = "ArticulatedObjectClass";
            std::string providerName;
        } properties;

        const std::string propertyPrefix = "mem.obj.articulated.";

        armem::client::Writer memoryWriter;
        std::mutex memoryWriterMutex;

        armem::client::Reader memoryReader;
        std::mutex memoryReaderMutex;

        // key: name of object: RobotDescription::name
        std::map<std::string, MemoryID> knownObjects;

        armem::ClientComponentPluginUser& component;
    };


}  // namespace armarx::armem::articulated_object