/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <cstdint>
#include <map>
#include <string>
#include <optional>
#include <mutex>

#include "ArmarXCore/core/application/properties/PropertyDefinitionContainer.h"

#include "ArmarXGui/libraries/RemoteGui/Client/Widgets.h"

#include <RobotAPI/interface/core/RobotState.h>

#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>
#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>

#include "RobotAPI/components/ArViz/Client/Client.h"

#include "RobotAPI/libraries/armem_objects/types.h"


namespace armarx::armem::server::obj::articulated_object_class
{
    class Segment;
}

namespace armarx::armem::server::obj::articulated_object_instance
{
    class Visu;

    class Segment : public armarx::Logging
    {
    public:

        struct CommitStats
        {
            int numUpdated = 0;
        };


        Segment(server::MemoryToIceAdapter& iceMemory,
                std::mutex& memoryMutex);

        virtual ~Segment();

        void connect(viz::Client arviz);

        void defineProperties(armarx::PropertyDefinitionsPtr& defs, const std::string& prefix = "");

        void init();

        void setArticulatedObjectClassSegment(const articulated_object_class::Segment& segment);

        ::armarx::armem::articulated_object::ArticulatedObjects getArticulatedObjects() const;

    private:

        server::MemoryToIceAdapter& iceMemory;
        wm::CoreSegment* coreSegment = nullptr;
        std::mutex& memoryMutex;

        articulated_object_class::Segment const*  classSegment;


        struct Properties
        {
            std::string coreInstanceSegmentName = "ArticulatedObjectInstance";
            int64_t maxHistorySize = -1;
        };
        Properties p;

        std::unique_ptr<Visu> visu;

    public:

        // struct RemoteGui
        // {
        //     armarx::RemoteGui::Client::GroupBox group;

        //     armarx::RemoteGui::Client::IntSpinBox maxHistorySize;
        //     armarx::RemoteGui::Client::CheckBox infiniteHistory;
        //     armarx::RemoteGui::Client::CheckBox discardSnapshotsWhileAttached;

        //     void setup(const Segment& data);
        //     void update(Segment& data);
        // };

    };

}  // namespace armarx::armem::server::obj::articulated_object_instance
