#pragma once

#include <mutex>
#include <string>

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <ArmarXGui/libraries/RemoteGui/Client/Widgets.h>

#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectID.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectFinder.h>

#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>
#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>
#include <RobotAPI/libraries/armem_objects/aron/ObjectClass.aron.generated.h>
#include <RobotAPI/libraries/armem_objects/server/class/FloorVis.h>


namespace armarx::armem::server::obj::clazz
{

    class Segment : public armarx::Logging
    {
    public:

        Segment(armem::server::MemoryToIceAdapter& iceMemory,
                std::mutex& memoryMutex);


        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");
        void init();
        void connect(viz::Client arviz);

        void loadByObjectFinder(const std::string& objectsPackage);
        void loadByObjectFinder(const ObjectFinder& finder);
        void loadByObjectFinder();

        void visualizeClass(const MemoryID& entityID, bool showAABB = true, bool showOOBB = true);


        static arondto::ObjectClass objectClassFromInfo(const ObjectInfo& info);


    private:

        armem::server::MemoryToIceAdapter& iceMemory;
        armem::wm::CoreSegment* coreSegment = nullptr;
        std::mutex& memoryMutex;

        ObjectFinder objectFinder;

        viz::Client arviz;
        FloorVis floorVis;


        struct Properties
        {
            std::string coreSegmentName = "Class";
            long maxHistorySize = -1;

            std::string objectsPackage = ObjectFinder::DefaultObjectsPackageName;
            bool loadFromObjectsPackage = true;
        };
        Properties p;


    public:

        struct RemoteGui
        {
            armarx::RemoteGui::Client::GroupBox group;

            struct Data
            {
                armarx::RemoteGui::Client::GroupBox group;

                armarx::RemoteGui::Client::IntSpinBox maxHistorySize;
                armarx::RemoteGui::Client::CheckBox infiniteHistory;

                void setup(const Segment& segment);
                void update(Segment& segment);
            };
            Data data;

            struct Visu
            {
                armarx::RemoteGui::Client::GroupBox group;

                std::vector<MemoryID> showOptionsIndex;
                armarx::RemoteGui::Client::ComboBox showComboBox;
                armarx::RemoteGui::Client::Button showButton;

                void setup(const Segment& segment);
                void update(Segment& segment);
            };
            Visu visu;

            void setup(const Segment& segment);
            void update(Segment& segment);

        };

    };

}
