/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <vector>

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <RobotAPI/libraries/armem_robot_state/common/localization/types.h>

namespace armarx::armem::wm
{
    class CoreSegment;
    class ProviderSegment;
} // namespace armarx::armem::wm

namespace armarx::armem::common::robot_state::localization
{
    using armarx::armem::common::robot_state::localization::TransformQuery;
    using armarx::armem::common::robot_state::localization::TransformResult;

    class TransformHelper
    {
      public:
        static TransformResult
        lookupTransform(const armem::wm::CoreSegment &localizationCoreSegment,
                        const TransformQuery &query);

      private:
        static std::vector<std::string>
        buildTransformChain(const armem::wm::CoreSegment &localizationCoreSegment,
                            const TransformQuery &query);

        static std::vector<Eigen::Affine3f>
        obtainTransforms(const armem::wm::CoreSegment &localizationCoreSegment,
                         const std::vector<std::string> &tfChain,
                         const std::string &agent,
                         const std::int64_t &timestamp);

        static Eigen::Affine3f
        obtainTransform(const std::string &entityName,
                        const armem::wm::ProviderSegment &agentProviderSegment,
                        int64_t timestamp);
    };
} // namespace armarx::armem::common::robot_state::localization